//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//DX.CPP - Code for the CDX class
//Copyright Marshall Harnish /97
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include "dx.h"
//#define _DEBUG_FILL_POLY_


//Resizes the texture map source size
int CDX::ResizeTS(int x, int y)
{
	TexturePoint[0].x=0;	//Setup the source texture coords
	TexturePoint[0].y=0;
	TexturePoint[1].x=x-1;
	TexturePoint[1].y=0;
	TexturePoint[2].x=x-1;
	TexturePoint[2].y=y-1;
	TexturePoint[3].x=0;
	TexturePoint[3].y=y-1;
	return(TRUE);
}


//*****************************************************
//Draws a texture mapped polygon on the screen
int CDX::FillTexturedPoly(int x1,int y1,int x2,int y2,int x3,int y3,int x4,int y4,int TileNum,int SrcSurface)
{
	LPSTR m_Bits,m_Source;
	CPoint Point[NUM_POLY_POINTS];
	int MaxY,MinY,MinI=0,MaxI=0,Index,Num,y,BitOffset;
	int ScnOffset;

	Point[0].x=x1;	//Setup the points
	Point[0].y=y1;
	Point[1].x=x2;
	Point[1].y=y2;
	Point[2].x=x3;
	Point[2].y=y3;
	Point[3].x=x4;
	Point[3].y=y4;

	MinY=Point[0].y;	//Set the min and max to default values
	MaxY=Point[0].y;

	//Setup the highest and lowest points on the polygon (for the Y coords)
	for(Index=0;Index<NUM_POLY_POINTS;Index++)
	{
		if(Point[Index].y < MinY)
			MinY=Point[MinI = Index].y;
		if(Point[Index].y > MaxY)
			MaxY=Point[MaxI = Index].y;
	}	
	
	//Check to make sure that the polygon isn't 0 in height
	if(MinY == MaxY)
		return FALSE;

	//////////////////////////////////////////
	//Get a pointer to the BACK surface memory and lock it
	if((m_Bits=LockSurfaceMem(CDX_BACK)) == NULL)
		return FALSE;	//Return if there was an error locking the surface memory
	//Get a pointer to the TILES surface memory
	if((m_Source=LockSurfaceMem(SrcSurface)) == NULL)
		return FALSE;
	
	//Figure out the offset of where the tile is
	m_Source+=(((TileNum/TILES_SCN_X)*TILE_SIZE)*SCN_SIZE_X)<<1;	//Add the Y coord onto the pointer
	m_Source+=((TileNum-((TileNum/TILES_SCN_X)*TILES_SCN_X))*TILE_SIZE)<<1;	//Add the X coord onto the pointer
	
	//Clip on the Y axis
	if(MinY < 0)	//Check to see if above 0
		MinY=0;
	if(MaxY > (SCN_SIZE_Y-1))	//Check to see if below
		MaxY=SCN_SIZE_Y-1;


	//Setup the scan edges (scan forward)
	Index=MinI;
	while(Index != MaxI)
	{
		Num=(Index+1)%NUM_POLY_POINTS;
		ScanEdge(fwdScan,&Point[Index],&Point[Num],&TexturePoint[Index],&TexturePoint[Num]);
		Index=Num;
	}

	//scan backward
	Index=MinI;
	while(Index != MaxI)
	{
		Num=(Index-1+NUM_POLY_POINTS)%NUM_POLY_POINTS;
		ScanEdge(bwdScan,&Point[Index],&Point[Num],&TexturePoint[Index],&TexturePoint[Num]);
		Index=Num;
	}

	ScnOffset=(SCN_SIZE_X<<1);	//Set the bytes offset for each scan line on the screen

	//Draw the scanlines
	BitOffset=(MinY*SCN_SIZE_X) << 1;	//Set the offset to the start of the bits to draw to (y*ScnSizeY*2)
	for(y=MinY;y<MaxY;y++)
	{
		if(bwdScan[y].x < fwdScan[y].x)
			DrawTexturedHLine(m_Bits+BitOffset,&bwdScan[y],&fwdScan[y],m_Source);
		else
			DrawTexturedHLine(m_Bits+BitOffset,&fwdScan[y],&bwdScan[y],m_Source);
		BitOffset+=ScnOffset;
	}

	///////////////////////////////////////////////
	//Unlock the surface memory
	UnlockSurfaceMem(CDX_BACK,m_Bits);
	UnlockSurfaceMem(SrcSurface,m_Source);
	return TRUE;
}

//*****************************************************
//Draws a single scanline on the screen (used
//internally by FillTexturedPoly
void CDX::DrawTexturedHLine(LPSTR m_DestBits,PolyEdgeScan *MinEdge,PolyEdgeScan *MaxEdge,LPSTR m_SrcBits)
{
	int MinX,MaxX,DeltaTX,DeltaTY,TextureX,TextureY,SrcOffset;
	int oldX,oldY;
	short int Color;	//Make color a 16bit number
	LPSTR CurrDest,EndDest;	//Pointers to the surface memory
	
	MinX=MinEdge->x;	//Setup the min and max x's
	MaxX=MaxEdge->x;

	if(MaxX <= MinX)		//Make sure the line goes from left to right
		return;
	
	//Setup the step through the src texture
	DeltaTX=(MaxEdge->TextureX - MinEdge->TextureX) / (MaxEdge->x - MinEdge->x);
	DeltaTY=(MaxEdge->TextureY - MinEdge->TextureY) / (MaxEdge->x - MinEdge->x);

	//Setup the X/Y coords in the src texture
	TextureX=MinEdge->TextureX;
	TextureY=MinEdge->TextureY;

	//Clip on the X axis
	if(MinX<0)
	{
		TextureX-=(MinX*DeltaTX);
		TextureY-=(MinX*DeltaTY);
		MinX=0;
	}
	if(MaxX>(SCN_SIZE_X-1))
		MaxX=SCN_SIZE_X;

	//Setup the pointers to the destination memory
	CurrDest=m_DestBits+(MinX<<1);
	EndDest=m_DestBits+(MaxX<<1);

	//Plot the line on the screen
	for(;CurrDest<EndDest;CurrDest+=2)
	{
		//Calculate the next color if the source image coords change
		if((oldX != (TextureX>>16)) || (oldY != (TextureY>>16)))
		{
			SrcOffset=(((TextureY>>16)*SCN_SIZE_X)<<1) + ((TextureX>>16)<<1);
			_asm {	//Get the new color
				mov edx,m_SrcBits		//Move the offset of the source bits into edx
				add edx,SrcOffset		//Add the offset
				mov ax,word ptr [edx]	//Move the color into ax (NOTE: 16bit regs are being used because we are in 16bpp mode)
				mov Color,ax			//Move ax into Color
			}
			oldX=TextureX>>16;
			oldY=TextureY>>16;
		}
		//Plot the pixel onto the surface mem
		_asm {
			mov edx,CurrDest		//Get the offset of the dest pt into edx
			mov ax,Color			//Move the Color into ax
			mov word ptr [edx],ax	//Copy the pixel over
		}
		TextureX+=DeltaTX;	//Add the DeltaX/Y to the source texture point x/y
		TextureY+=DeltaTY;
	}	
}

//*****************************************************
//Scans the edges of a polygon
void CDX::ScanEdge(PolyEdgeScan *Edge,CPoint *DestA,CPoint *DestB,CPoint *TextureA,CPoint *TextureB)
{
	int MinY,MaxY,DeltaX,DeltaTX,DeltaTY;
	int CurrX,TextureX,TextureY;
	
	MinY=DestA->y;
	MaxY=DestB->y;
	if(MaxY < MinY)	//Return if the max is less that the min
		return;
	
	//NOTE: All numbers are shifted left by 16 so that floating point numbers wont have to be used!
	if(DestB->y != DestA->y)	//Check to make sure a divide by 0 doesnt happen!
	{
		DeltaX=(((DestB->x - DestA->x)<<16) / (DestB->y - DestA->y));
		DeltaTX=(((TextureB->x - TextureA->x)<<16) / (DestB->y - DestA->y));
		DeltaTY=(((TextureB->y - TextureA->y)<<16) / (DestB->y - DestA->y));
	}
	else	//Else, set to 0
	{
		DeltaX=0;
		DeltaTX=0;
		DeltaTY=0;
	}

	//Setup the starting values
	CurrX=(DestA->x << 16);
	TextureX=(TextureA->x << 16);
	TextureY=(TextureA->y << 16);


	//Clip on the Y axis
	if(MinY<0)		//Check for <0
	{
		MinY=0;
		CurrX-=(DestA->y*DeltaX);
		TextureX-=(DestA->y*DeltaTX);
		TextureY-=(DestA->y*DeltaTY);
	}
	if(MaxY > (SCN_SIZE_Y-1))	//Check for >(SCN_SIZE_Y-1)
		MaxY=SCN_SIZE_Y-1;

	//Interpolate the edges (get the corresponding x,tx,ty points)
	for(;MinY<MaxY;MinY++)
	{
		Edge[MinY].x = (CurrX>>16);
		Edge[MinY].TextureX = TextureX;
		Edge[MinY].TextureY = TextureY;
		CurrX+=DeltaX;
		TextureX+=DeltaTX;
		TextureY+=DeltaTY;
	}
}

//*****************************************************
//Locks a surface and returns a pointer to the memory
LPSTR CDX::LockSurfaceMem(int SurfaceNum)
{
	DDSURFACEDESC SurfaceDesc;		//Description of a surface (used by lock/unlock)
	LPDIRECTDRAWSURFACE Surface;

	switch(SurfaceNum)		//Determine which surfaces the user wants to lock
	{
	case CDX_BACK:
		Surface=m_Back;
		break;
	case CDX_WORK:
		Surface=m_Work;
		break;
	case CDX_TILES:
		Surface=m_Tiles;
		break;
	}

	//Lock the back buffer so that we can write directly to the memory
	SurfaceDesc.dwSize=sizeof(SurfaceDesc);		//Set the size of the struct
	SurfaceDesc.dwFlags=0;	//Clear all of the flags
	if((Surface->Lock(NULL,&SurfaceDesc,DDLOCK_SURFACEMEMORYPTR|DDLOCK_WAIT|DDLOCK_NOSYSLOCK,NULL)) != DD_OK)
	{
		TRACE("* Couldn't LOCK the surface!\n");
		return NULL;
	}

	//Return a pointer to the memory if everything went ok
	return (LPSTR)SurfaceDesc.lpSurface;
}

//*****************************************************
//Unlocks a surface
void CDX::UnlockSurfaceMem(int SurfaceNum,LPSTR SurfaceMem)
{
	LPDIRECTDRAWSURFACE Surface;

	switch(SurfaceNum)		//Determine which surface the user wants to unlock
	{
	case CDX_BACK:
		Surface=m_Back;
		break;
	case CDX_WORK:
		Surface=m_Work;
		break;
	case CDX_TILES:
		Surface=m_Tiles;
		break;
	}

	//Unlock the surface and return
	Surface->Unlock(SurfaceMem);
}

//*****************************************************
//Fills an area on a surface with a color
int CDX::FillArea(CRect *DestRect,DWORD Color,int Surface)
{
	LPDIRECTDRAWSURFACE DstSurface;
	DDBLTFX BltFX;

	//Get the correct surface to draw to
	switch(Surface)
	{
	case CDX_BACK:
		DstSurface=m_Back;
		break;
	case CDX_TILES:
		DstSurface=m_Tiles;
		break;
	}

	BltFX.dwSize=sizeof(BltFX);
	BltFX.dwFillColor=Color;

	//Fill the area
	DstSurface->Blt(DestRect,NULL,NULL,DDBLT_COLORFILL|DDBLT_WAIT,&BltFX);
	return TRUE;
}


//*****************************************************
//Copies a section of one surface to another
int CDX::CopyArea(CRect *SourceRect,CPoint *DestPt,int SourceSurface,int DestSurface)
{
	LPDIRECTDRAWSURFACE SrcSurface;		//Create 2 pointers to the surfaces
	LPDIRECTDRAWSURFACE DstSurface;

	//Setup the source surface to blit FROM
	switch(SourceSurface)
	{
	case CDX_BACK:
		SrcSurface=m_Back;
		break;
	case CDX_WORK:
		SrcSurface=m_Work;
		break;
	case CDX_TILES:
		SrcSurface=m_Tiles;
		break;
	}

	//Setup the destination surface to blit TO
	switch(DestSurface)
	{
	case CDX_BACK:
		DstSurface=m_Back;
		break;
	case CDX_WORK:
		DstSurface=m_Work;
		break;
	case CDX_TILES:
		DstSurface=m_Tiles;
		break;
	}

	//Blit it
	DstSurface->BltFast(DestPt->x,DestPt->y,SrcSurface,SourceRect,DDBLTFAST_SRCCOLORKEY|DDBLTFAST_WAIT);
	//DstSurface->BltFast(DestPt->x,DestPt->y,SrcSurface,SourceRect,DDBLTFAST_NOCOLORKEY|DDBLTFAST_WAIT);

	return TRUE;
}

//*****************************************************
//Loads a bitmap onto the given buffer and displays it
int CDX::LoadBitmap(char *iFileName,int Surface,int x,int y)
{
	CFile iFile;				//The file to be opened
	BITMAPFILEHEADER fileHeader;
	BITMAPINFOHEADER infoHeader;
	BITMAPINFO *m_Info;			//Bitmapinfo object for the bitmap file
	BYTE *m_Pixels;				//Pointer to the bytes in the bitmap file
	HDC hSurface;				//Handle to the surface
	int cPaletteEntries,cColorTable,cInfo,cPixels;

	//**************************************
	//* Read the bitmap into the buffer
	//Open the file
	if(!(iFile.Open(iFileName,CFile::modeRead)))
	{
		TRACE("* Couldn't open the file: %s\n",iFileName);
		return FALSE;
	}
	//Read in the bitmap file header (into the temporary buffer) and check it for the correct header (BM)
	iFile.Read(&fileHeader,sizeof(BITMAPFILEHEADER));
	if(fileHeader.bfType != 0x4D42)
	{
		TRACE("* %s is not a valid bitmap file!\n",iFileName);
		iFile.Close();
		return FALSE;
	}
	//Read in the information header from the bitmap file
	iFile.Read(&infoHeader,sizeof(BITMAPINFOHEADER));
	if(infoHeader.biSize != sizeof(BITMAPINFOHEADER))
	{
		TRACE("* OS/2 bitmap format not supported!\n");
		iFile.Close();
		return FALSE;
	}

	//Initialize various data
	if((cPaletteEntries=infoHeader.biClrUsed) == 0)		//If the number of colors is stated to be 0 in the bitmap file, then:
		cPaletteEntries=(1 << infoHeader.biBitCount);	//Shift the number 1 by the number of bits per pixel to get the number of colors
	cColorTable=256*sizeof(RGBQUAD);	//Size of palette (256 colors * (RGB bytes) = 768)
	cInfo=sizeof(BITMAPINFOHEADER)+cColorTable;	//Size of the BITMAPINFOHEADER struct and the palette
	cPixels=fileHeader.bfSize;			//Store the number of pixels in the bitmap

	//Save the bitmap info header in m_Info
	m_Info=(BITMAPINFO *)new BYTE[cInfo];	//Declare enough room in memory to hold a BITMAPINFOHEADER struct
	memcpy(m_Info,&infoHeader,sizeof(BITMAPINFOHEADER));	//Copy the bitmap info header into m_Info
	iFile.Read((BYTE *)m_Info+sizeof(BITMAPINFOHEADER),cColorTable);	//Read in the palette

	//Allocate space for the pixels and store them in m_Pixels
	m_Pixels=new BYTE[cPixels];		//Allocate the room in memory
	iFile.Seek(fileHeader.bfOffBits,CFile::begin);	//Seek to the start of the pixels in the file

	if(m_Pixels != NULL)
		iFile.Read(m_Pixels,cPixels);	//Read the pixels into m_Pixels

	iFile.Close();		//Close the file

	//*******************************************
	//* Display the bitmap on the given surface
	switch(Surface)
	{
	case CDX_BACK:
		m_Back->GetDC(&hSurface);		//Get the DC for the back surface
		//Draw the bitmap on the back surface
		if(StretchDIBits(hSurface,x,y,infoHeader.biWidth,infoHeader.biHeight,0,0,infoHeader.biWidth,infoHeader.biHeight,m_Pixels,m_Info,DIB_RGB_COLORS,SRCCOPY) == GDI_ERROR)
			TRACE("* Couldn't draw the bitmap on the back surface!\n");
		m_Back->ReleaseDC(hSurface);	//Release the DC (VERY important!)
		break;
	case CDX_TILES:
		m_Tiles->GetDC(&hSurface);		//Get the DC for the tiles surface
		//Draw the bitmap on the tiles surface
		if(StretchDIBits(hSurface,x,y,infoHeader.biWidth,infoHeader.biHeight,0,0,infoHeader.biWidth,infoHeader.biHeight,m_Pixels,m_Info,DIB_RGB_COLORS,SRCCOPY) == GDI_ERROR)
			TRACE("* Couldn't draw the bitmap on the tiles surface!\n");
		m_Tiles->ReleaseDC(hSurface);	//Release the DC (VERY important!)
		break;
	}

	//*******************************
	//* Delete the temporary varibles
	if(m_Pixels != NULL)		//Clean up the bitmap info/pixel buffers
		delete [] m_Pixels;
	if(m_Info != NULL)
		delete [] (BYTE *)m_Info;

	return TRUE;
}

//*****************************************************
//Flips the back surface onto the primary
int CDX::FlipSurfaces(void)
{
	//Check to make sure that none of the surfaces were lost
	if((m_Primary->IsLost()) == DDERR_SURFACELOST)	//If so, restore them
		m_Primary->Restore();

	//Flip the back surface onto the screen (into the primary surface)
#ifdef _FULL_SCREEN_
	if((m_Primary->Flip(NULL,DDFLIP_WAIT)) != DD_OK)
	{
		TRACE("* Couldn't flip the surfaces!\n");
		return FALSE;
	}
#else
	POINT DestPt;
	RECT Source,Dest;
	DestPt.x=0;
	DestPt.y=0;
	ClientToScreen(AfxGetMainWnd()->GetSafeHwnd(),&DestPt);
	AfxGetMainWnd()->GetClientRect(&Dest);
	OffsetRect(&Dest,DestPt.x,DestPt.y);
	SetRect(&Source,0,0,SCN_SIZE_X,SCN_SIZE_Y);
	if((m_Primary->Blt(&Dest,m_Back,&Source,DDBLT_WAIT,NULL)) != DD_OK)
	{
		TRACE("* Couldn't flip the surfaces!\n");
		return FALSE;
	}
#endif

	//If it flipped, return true
	return TRUE;
}

//*****************************************************
//Initializes the DD surfaces
int CDX::InitSurfaces(void)
{
	int hr;
	DDSURFACEDESC DDSDesc;	//Create a surface description
	DDSDesc.dwSize=sizeof(DDSDesc);	//Set the size of the structure
#ifdef _FULL_SCREEN_
	DDSDesc.dwFlags=DDSD_BACKBUFFERCOUNT|DDSD_CAPS;	//Set the valid flags (in this case, for more than 1 back buffer and for the caps struct to be valid)
	DDSDesc.ddsCaps.dwCaps=DDSCAPS_PRIMARYSURFACE|DDSCAPS_FLIP|DDSCAPS_COMPLEX;	//Setup the surface to be the primary surface, with a complex flipping structure
	DDSDesc.dwBackBufferCount=1;	//Tell it to create 1 back buffer
#else
	DDSDesc.dwFlags=DDSD_CAPS;	//Set the valid flags (in this case, for more than 1 back buffer and for the caps struct to be valid)
	DDSDesc.ddsCaps.dwCaps=DDSCAPS_PRIMARYSURFACE;	//Setup the surface to be the primary surface, with a complex flipping structure
#endif

	//Create the primary surface
	if((hr=m_pDD->CreateSurface(&DDSDesc,&m_Primary,NULL)) != DD_OK)
	{
		TRACE("* Couldn't create the PRIMARY surface!\n");
		TraceErrorDD(hr);
		return FALSE;
	}

	//Get a pointer to the 1 back surface that was created
#ifdef _FULL_SCREEN_
	DDSDesc.dwFlags=DDSD_CAPS;
	DDSDesc.ddsCaps.dwCaps=DDSCAPS_BACKBUFFER;
	if((hr=m_Primary->GetAttachedSurface(&DDSDesc.ddsCaps,&m_Back)) != DD_OK)
#else
	DDSDesc.dwFlags=DDSD_CAPS|DDSD_WIDTH|DDSD_HEIGHT;
	DDSDesc.dwWidth=SCN_SIZE_X;
	DDSDesc.dwHeight=SCN_SIZE_Y;
	DDSDesc.ddsCaps.dwCaps=DDSCAPS_OFFSCREENPLAIN;	//Set it up to be the back buffer
	if((hr=m_pDD->CreateSurface(&DDSDesc,&m_Back,NULL)) != DD_OK)
#endif
	{
		TRACE("* Couldn't get the back buffer!\n");
		TraceErrorDD(hr);
		return FALSE;
	}

	//Create another surface for the tiles (NOT part of the flipping structure for the other 2)
	//DDSDesc.dwFlags=DDSD_CAPS|DDSD_WIDTH|DDSD_HEIGHT;
	DDSDesc.ddsCaps.dwCaps=DDSCAPS_OFFSCREENPLAIN;
	DDSDesc.dwFlags=DDSD_CAPS|DDSD_WIDTH|DDSD_HEIGHT|DDSD_CKSRCBLT;
	DDSDesc.ddckCKSrcBlt.dwColorSpaceLowValue=0;
	DDSDesc.ddckCKSrcBlt.dwColorSpaceHighValue=0;
	DDSDesc.dwWidth=SCN_SIZE_X;
	DDSDesc.dwHeight=SCN_SIZE_Y;
	if((m_pDD->CreateSurface(&DDSDesc,&m_Tiles,NULL)) != DD_OK)
	{
		TRACE("* Couldn't create the TILES surface!\n");
		return FALSE;
	}
	
	//creates the m_Work surface
	if((m_pDD->CreateSurface(&DDSDesc,&m_Work,NULL)) != DD_OK)
	{
		TRACE("* Couldn't create the WORK surface!\n");
		return FALSE;
	}

	//The surfaces were created, return
	return TRUE;
}

//*****************************************************
//Initializes DD and sets the video mode to the given values
int CDX::InitDD(int ScnSizeX,int ScnSizeY,int BPP)
{
	//Create the primary DD object
	if(DirectDrawCreate(NULL,&m_pDD,NULL) != DD_OK)
	{
		TRACE("* Couldn't initialize the primary DD object!\n");
		return FALSE;
	}
	//Set the cooperative level of the screen with Windows (required for Windows to give us access to the screen)
#ifdef _FULL_SCREEN_
	if((m_pDD->SetCooperativeLevel(AfxGetMainWnd()->GetSafeHwnd(),DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN)) != DD_OK)
	{
		TRACE("* Couldn't set the cooperative level of the screen with Windows! (FULL SCREEN)\n");
		return FALSE;
	}
	//Set the display mode to the given values
	if((m_pDD->SetDisplayMode(ScnSizeX,ScnSizeY,BPP)) != DD_OK)
	{
		TRACE("* Couldn't set the display mode to: %dx%dx%d!\n",ScnSizeX,ScnSizeY,BPP);
		return FALSE;
	}
#else
	if((m_pDD->SetCooperativeLevel(AfxGetMainWnd()->GetSafeHwnd(),DDSCL_NORMAL)) != DD_OK)
	{
		TRACE("* Couldn't set the cooperative level of the screen with Windows (WINDOW)!\n");
		return FALSE;
	}
#endif
	//Everything initialized ok, return
	return TRUE;
}

//*****************************************************
//The program must call this to create the CDX object (unless the user wants to call the 2 routines seperately for some reason)
int CDX::Create(int Width,int Height,int BPP)
{
	if(!InitDD(Width,Height,BPP))	//Init DD
		return FALSE;
	if(!InitSurfaces())				//Init the surfaces
		return FALSE;
	
	return TRUE;	//Everything went ok
}

//*****************************************************
//The constructor
CDX::CDX()
{
	fwdScan = new PolyEdgeScan[SCN_SIZE_Y];	//Scanline edges... used by FillTexturedPoly()
	bwdScan = new PolyEdgeScan[SCN_SIZE_Y];
	TexturePoint[0].x=0;	//Setup the source texture coords
	TexturePoint[0].y=0;
	TexturePoint[1].x=TILE_SIZE-1;
	TexturePoint[1].y=0;
	TexturePoint[2].x=TILE_SIZE-1;
	TexturePoint[2].y=TILE_SIZE-1;
	TexturePoint[3].x=0;
	TexturePoint[3].y=TILE_SIZE-1;
}

//*****************************************************
//The destructor
CDX::~CDX()
{
	if(fwdScan)	//Cleanup the memory for storing the scanlines
		delete [] fwdScan;
	if(bwdScan)
		delete [] bwdScan;
}

//*****************************************************
//Traces a DirectDraw error (can be very useful!)
void CDX::TraceErrorDD(HRESULT hErr)
{       
    switch (hErr)
    {
        case DDERR_ALREADYINITIALIZED:
                TRACE("DDERR_ALREADYINITIALIZED"); break;
        case DDERR_CANNOTATTACHSURFACE:
                TRACE("DDERR_CANNOTATTACHSURFACE"); break;
        case DDERR_CANNOTDETACHSURFACE:
                TRACE("DDERR_CANNOTDETACHSURFACE"); break;
        case DDERR_CURRENTLYNOTAVAIL:
                TRACE("DDERR_CURRENTLYNOTAVAIL"); break;
        case DDERR_EXCEPTION:
                TRACE("DDERR_EXCEPTION"); break;
        case DDERR_GENERIC:
                TRACE("DDERR_GENERIC"); break;
        case DDERR_HEIGHTALIGN:
                TRACE("DDERR_HEIGHTALIGN"); break;
        case DDERR_INCOMPATIBLEPRIMARY:
                TRACE("DDERR_INCOMPATIBLEPRIMARY"); break;
        case DDERR_INVALIDCAPS:
                TRACE("DDERR_INVALIDCAPS"); break;
        case DDERR_INVALIDCLIPLIST:
                TRACE("DDERR_INVALIDCLIPLIST"); break;
        case DDERR_INVALIDMODE:
                TRACE("DDERR_INVALIDMODE"); break;
        case DDERR_INVALIDOBJECT:
                TRACE("DDERR_INVALIDOBJECT"); break;
        case DDERR_INVALIDPARAMS:
                TRACE("DDERR_INVALIDPARAMS"); break;
        case DDERR_INVALIDPIXELFORMAT:
                TRACE("DDERR_INVALIDPIXELFORMAT"); break;
        case DDERR_INVALIDRECT:
                TRACE("DDERR_INVALIDRECT"); break;
        case DDERR_LOCKEDSURFACES:
                TRACE("DDERR_LOCKEDSURFACES"); break;
        case DDERR_NO3D:
                TRACE("DDERR_NO3D"); break;
        case DDERR_NOALPHAHW:
                TRACE("DDERR_NOALPHAHW"); break;
        case DDERR_NOCLIPLIST:
                TRACE("DDERR_NOCLIPLIST"); break;
        case DDERR_NOCOLORCONVHW:
                TRACE("DDERR_NOCOLORCONVHW"); break;
        case DDERR_NOCOOPERATIVELEVELSET:
                TRACE("DDERR_NOCOOPERATIVELEVELSET"); break;
        case DDERR_NOCOLORKEY:
                TRACE("DDERR_NOCOLORKEY"); break;
        case DDERR_NOCOLORKEYHW:
                TRACE("DDERR_NOCOLORKEYHW"); break;
        case DDERR_NODIRECTDRAWSUPPORT:
                TRACE("DDERR_NODIRECTDRAWSUPPORT"); break;
        case DDERR_NOEXCLUSIVEMODE:
                TRACE("DDERR_NOEXCLUSIVEMODE"); break;
        case DDERR_NOFLIPHW:
                TRACE("DDERR_NOFLIPHW"); break;
        case DDERR_NOGDI:
                TRACE("DDERR_NOGDI"); break;
        case DDERR_NOMIRRORHW:
                TRACE("DDERR_NOMIRRORHW"); break;
        case DDERR_NOTFOUND:
                TRACE("DDERR_NOTFOUND"); break;
        case DDERR_NOOVERLAYHW:
                TRACE("DDERR_NOOVERLAYHW"); break;
        case DDERR_NORASTEROPHW:
                TRACE("DDERR_NORASTEROPHW"); break;
        case DDERR_NOROTATIONHW:
                TRACE("DDERR_NOROTATIONHW"); break;
        case DDERR_NOSTRETCHHW:
                TRACE("DDERR_NOSTRETCHHW"); break;
        case DDERR_NOT4BITCOLOR:
                TRACE("DDERR_NOT4BITCOLOR"); break;
        case DDERR_NOT4BITCOLORINDEX:
                TRACE("DDERR_NOT4BITCOLORINDEX"); break;
        case DDERR_NOT8BITCOLOR:
                TRACE("DDERR_NOT8BITCOLOR"); break;
        case DDERR_NOTEXTUREHW:
                TRACE("DDERR_NOTEXTUREHW"); break;
        case DDERR_NOVSYNCHW:
                TRACE("DDERR_NOVSYNCHW"); break;
        case DDERR_NOZBUFFERHW:
                TRACE("DDERR_NOZBUFFERHW"); break;
        case DDERR_NOZOVERLAYHW:
                TRACE("DDERR_NOZOVERLAYHW"); break;
        case DDERR_OUTOFCAPS:
                TRACE("DDERR_OUTOFCAPS"); break;
        case DDERR_OUTOFMEMORY:
                TRACE("DDERR_OUTOFMEMORY"); break;
        case DDERR_OUTOFVIDEOMEMORY:
                TRACE("DDERR_OUTOFVIDEOMEMORY"); break;
        case DDERR_OVERLAYCANTCLIP:
                TRACE("DDERR_OVERLAYCANTCLIP"); break;
        case DDERR_OVERLAYCOLORKEYONLYONEACTIVE:
                TRACE("DDERR_OVERLAYCOLORKEYONLYONEACTIVE"); break;
        case DDERR_PALETTEBUSY:
                TRACE("DDERR_PALETTEBUSY"); break;
        case DDERR_COLORKEYNOTSET:
                TRACE("DDERR_COLORKEYNOTSET"); break;
        case DDERR_SURFACEALREADYATTACHED:
                TRACE("DDERR_SURFACEALREADYATTACHED"); break;
        case DDERR_SURFACEALREADYDEPENDENT:
                TRACE("DDERR_SURFACEALREADYDEPENDENT"); break;
        case DDERR_SURFACEBUSY:
                TRACE("DDERR_SURFACEBUSY"); break;
        case DDERR_CANTLOCKSURFACE:
                TRACE("DDERR_CANTLOCKSURFACE"); break;
        case DDERR_SURFACEISOBSCURED:
                TRACE("DDERR_SURFACEISOBSCURED"); break;
        case DDERR_SURFACELOST:
                TRACE("DDERR_SURFACELOST"); break;
        case DDERR_SURFACENOTATTACHED:
                TRACE("DDERR_SURFACENOTATTACHED"); break;
        case DDERR_TOOBIGHEIGHT:
                TRACE("DDERR_TOOBIGHEIGHT"); break;
        case DDERR_TOOBIGSIZE:
                TRACE("DDERR_TOOBIGSIZE"); break;
        case DDERR_TOOBIGWIDTH:
                TRACE("DDERR_TOOBIGWIDTH"); break;
        case DDERR_UNSUPPORTED:
                TRACE("DDERR_UNSUPPORTED"); break;
        case DDERR_UNSUPPORTEDFORMAT:
                TRACE("DDERR_UNSUPPORTEDFORMAT"); break;
        case DDERR_UNSUPPORTEDMASK:
                TRACE("DDERR_UNSUPPORTEDMASK"); break;
        case DDERR_VERTICALBLANKINPROGRESS:
                TRACE("DDERR_VERTICALBLANKINPROGRESS"); break;
        case DDERR_WASSTILLDRAWING:
                TRACE("DDERR_WASSTILLDRAWING"); break;
        case DDERR_XALIGN:
                TRACE("DDERR_XALIGN"); break;
        case DDERR_INVALIDDIRECTDRAWGUID:
                TRACE("DDERR_INVALIDDIRECTDRAWGUID"); break;
        case DDERR_DIRECTDRAWALREADYCREATED:
                TRACE("DDERR_DIRECTDRAWALREADYCREATED"); break;
        case DDERR_NODIRECTDRAWHW:
                TRACE("DDERR_NODIRECTDRAWHW"); break;
        case DDERR_PRIMARYSURFACEALREADYEXISTS:
                TRACE("DDERR_PRIMARYSURFACEALREADYEXISTS"); break;
        case DDERR_NOEMULATION:
                TRACE("DDERR_NOEMULATION"); break;
        case DDERR_REGIONTOOSMALL:
                TRACE("DDERR_REGIONTOOSMALL"); break;
        case DDERR_CLIPPERISUSINGHWND:
                TRACE("DDERR_CLIPPERISUSINGHWND"); break;
        case DDERR_NOCLIPPERATTACHED:
                TRACE("DDERR_NOCLIPPERATTACHED"); break;
        case DDERR_NOHWND:
                TRACE("DDERR_NOHWND"); break;
        case DDERR_HWNDSUBCLASSED:
                TRACE("DDERR_HWNDSUBCLASSED"); break;
        case DDERR_HWNDALREADYSET:
                TRACE("DDERR_HWNDALREADYSET"); break;
        case DDERR_NOPALETTEATTACHED:
                TRACE("DDERR_NOPALETTEATTACHED"); break;
        case DDERR_NOPALETTEHW:
                TRACE("DDERR_NOPALETTEHW"); break;
        case DDERR_BLTFASTCANTCLIP:
                TRACE("DDERR_BLTFASTCANTCLIP"); break;
        case DDERR_NOBLTHW:
                TRACE("DDERR_NOBLTHW"); break;
        case DDERR_NODDROPSHW:
                TRACE("DDERR_NODDROPSHW"); break;
        case DDERR_OVERLAYNOTVISIBLE:
                TRACE("DDERR_OVERLAYNOTVISIBLE"); break;
        case DDERR_NOOVERLAYDEST:
                TRACE("DDERR_NOOVERLAYDEST"); break;
        case DDERR_INVALIDPOSITION:
                TRACE("DDERR_INVALIDPOSITION"); break;
        case DDERR_NOTAOVERLAYSURFACE:
                TRACE("DDERR_NOTAOVERLAYSURFACE"); break;
        case DDERR_EXCLUSIVEMODEALREADYSET:
                TRACE("DDERR_EXCLUSIVEMODEALREADYSET"); break;
        case DDERR_NOTFLIPPABLE:
                TRACE("DDERR_NOTFLIPPABLE"); break;
        case DDERR_CANTDUPLICATE:
                TRACE("DDERR_CANTDUPLICATE"); break;
        case DDERR_NOTLOCKED:
                TRACE("DDERR_NOTLOCKED"); break;
        case DDERR_CANTCREATEDC:
                TRACE("DDERR_CANTCREATEDC"); break;
        case DDERR_NODC:
                TRACE("DDERR_NODC"); break;
        case DDERR_WRONGMODE:
                TRACE("DDERR_WRONGMODE"); break;
        case DDERR_IMPLICITLYCREATED:
                TRACE("DDERR_IMPLICITLYCREATED"); break;
        case DDERR_NOTPALETTIZED:
                TRACE("DDERR_NOTPALETTIZED"); break;
        case DDERR_UNSUPPORTEDMODE:
                TRACE("DDERR_UNSUPPORTEDMODE"); break;
        case DDERR_NOMIPMAPHW:
                TRACE("DDERR_NOMIPMAPHW"); break;
        case DDERR_INVALIDSURFACETYPE:
                TRACE("DDERR_INVALIDSURFACETYPE"); break;
        case DDERR_DCALREADYCREATED:
                TRACE("DDERR_DCALREADYCREATED"); break;
        case DDERR_CANTPAGELOCK:
                TRACE("DDERR_CANTPAGELOCK"); break;
        case DDERR_CANTPAGEUNLOCK:
                TRACE("DDERR_CANTPAGEUNLOCK"); break;
        case DDERR_NOTPAGELOCKED:
                TRACE("DDERR_NOTPAGELOCKED"); break;
        case DDERR_NOTINITIALIZED:
                TRACE("DDERR_NOTINITIALIZED"); break;
        default:
                TRACE("Unknown Error"); break;
        }
        TRACE("\n");
}